from typing import Protocol, Callable, Any
from DAP.Core import NodeAddr
from DAP.Crypto import GUUID
from DAP.Network import ClusterRole
from DAP.Network import Member

class DB(Protocol):
    @staticmethod
    def get(key : str, group : str) -> bytes: ...
    @staticmethod
    def set(key : str, group : str, value : bytes, isPinned : bool | None = None) -> bool: ...
    @staticmethod
    def set_sync(key : str, group : str, value : bytes, isPinned : bool | None = None) -> bool: ...
    @staticmethod
    def delete(key : str, group : str) -> bool: ...
    @staticmethod
    def pin(key : str, group : str) -> bool: ...
    @staticmethod
    def unpin(key : str, group : str) -> bool: ...
    @staticmethod
    def grLoad(group : str) -> list: ...

class Container(Protocol):
    @property
    def key(self) -> str: ...
    @property
    def value(self) -> bytes: ...

class Instance(Protocol):
    def __init__(self) -> None: ...
    pass

class Cluster(Protocol):
    def __init__(self, dbi : Instance, memberName : str, GUUID : GUUID, groupMask : str, ttl : int, ownerRootAccess : bool, 
                 defaultRole : MemberRole, linksClusterRole : ClusterRole) -> None: ...
    def memberAdd(self, address : NodeAddr, role : MemberRole) -> Member: ...
    def memberDelete(self, address : NodeAddr) -> bool: ...
    def notifyAdd(self, func : [str, str, str, bytes, Any], callback_args) -> None: ...
    def AddNetAssociate(self, net_id : int) -> bool: ...
    @staticmethod
    def byGroup(instance: Instance, groupName : str) -> Cluster: ...

class MemberRole(Protocol):
    def __eq__(self, value: MemberRole) -> bool: ...
    def __ne__(self, value: MemberRole) -> bool: ...

class MemberRoles(Protocol):
    @property
    def NOBODY(self) -> MemberRole: ...
    @property
    def GUEST(self) -> MemberRole: ...
    @property
    def USER(self) -> MemberRole: ...
    @property
    def ROOT(self) -> MemberRole: ...
    @property
    def DEFAULT(self) -> MemberRole: ...
    @property
    def INVALID(self) -> MemberRole: ...