from pydantic import BaseModel, Field, field_validator, ValidationInfo
from eth_typing import HexStr, HexAddress
from web3.types import HexBytes


class EventData(BaseModel):
    @classmethod
    def model_validate(cls, *args, **kwargs):
        base_data = dict(args[0])
        for key, value in base_data['args'].items():
            base_data[key] = value
        return super().model_validate(*(base_data,), **kwargs)


class LockedEventData(EventData):
    time: int
    sender: str
    value: int
    wallet_cn: str = Field(..., alias="walletCN")
    token_address: str = Field(..., alias="tokenAddress")
    token_symbol: str
    network_id: int = Field(..., alias="networkId")
    block: int = Field(..., alias="_block")
    transaction_hash: str = Field(..., alias="transactionHash")

    @field_validator("value", mode="before")
    def set_value(cls, value: int, fields: ValidationInfo):
        if fields.data.get("token_symbol") == "USDC":
            value = value * 10 ** 12
        return value

    @field_validator("transactionHash", mode="before")
    def get_tx_hash(cls, tx_hash: HexBytes):
        return tx_hash.hex()


class UnlockEventData(EventData):
    pass


class ConsensusEventData(EventData):
    pass


class StakedEventData(EventData):
    time: int
    sender: str
    value: int
    wallet_cn: str = Field(..., alias="walletCN")
    token: str
    network_id: int = Field(..., alias="networkID")
    count_months: int = Field(..., alias="countMonths")
    time_unlock: int = Field(..., alias="timeUnlock")
    tx_id: int = Field(..., alias="txId")
    procentage: int
